
import telnetlib

import sys, getopt
import time
import os,  termios, fcntl

from beanstalkdstats import BeanstalkdStats
from beanstalkd import Beanstalkd
from console import Console
from clock import Clock
from charreader import CharReader
from screenprinter import ScreenPrinter
from arguments import Arguments



def create_arguments_parser():
    return Arguments(sys, getopt)
    
def start_application(host, port):
    telnet = telnetlib.Telnet()
    beanstalkd = Beanstalkd(telnet, host, port)
    stats = BeanstalkdStats(beanstalkd)
    char_reader = CharReader(os, sys, termios, fcntl)
    screen_printer = ScreenPrinter(os, sys)
    console = Console(Clock(time),  char_reader,  screen_printer, stats)       
    return beanstalkd, console



import unittest

from mocks import *
from beantop.console import Console

class Test(unittest.TestCase):
    def setUp(self):
        self.screen = MockScreen()
        self.time = MockIO()
        self.char_reader = MockIO()
        self.screen_printer = MockIO()
        self.console = Console(self.time, self.char_reader,  self.screen_printer, self.screen)
        
    def test_mainloop(self):
        self.char_reader.put_char(list("asdfr")+[None]*10+list("ffasq"))
        self.console.main_loop()
        self.assertEquals(11,  self.time.totalsleep)
        self.assertEquals(3,  self.screen.printcalls)

if __name__ == "__main__":
    unittest.main() 




class Clock:
    def __init__(self,  time_library):
        self._time_library = time_library
    
    def gmtime(self):
        return self._time_library.time()
        
    def sleep(self, secs):
        return self._time_library.sleep(secs)
        
    def get_printable_time(self):
        return self._time_library.strftime("%a, %d %b %Y %X", 
                                          self._time_library.gmtime())



import unittest

from beantop.screenprinter import ScreenPrinter
from mock import *
   
class Test(unittest.TestCase):
    def setUp(self):
        self.os=Mock()
        self.file=Mock(write=MagicMock())
        self.sys=Mock(stdout=self.file)
        self.screen_printer = ScreenPrinter(self.os,  self.sys)
    
    def test_print_lines(self):
        lines = ['a', 'b']
        self.screen_printer.print_lines(lines)
        self.file.write.assert_has_calls([call('a\n'), call('b\n')])

    def test_clear(self):
        self.screen_printer.clear()
        self.assertTrue(self.os.wasClearCalled)
        self.os.clear.assert_called()
        


if __name__ == "__main__":
    unittest.main() 




import yaml

class Beanstalkd:
    def __init__(self,  conn,  host,  port):
        self._conn = conn       
        self._host = host
        self._port = port
     
    def connect(self):
        self._conn.open(self._host, self._port)

    def yaml_data_filtered(self, msg, fields_to_show):
        stats =  self.yaml_data(msg)
        ret_data = dict()
        for field in fields_to_show:
            ret_data[field] = stats[field]
        return ret_data
        
    def yaml_data(self, msg):
        self._send(msg)
        self._read_line()      
        self._read_line()
        stats =  yaml.load(self._conn.read_until("\r\n"))
        return stats

    def _send(self, mess):
        self._conn.write(mess+"\r\n")
      
    def _read_line(self):
        return self._conn.read_until("\n")    
      



import unittest

from mocks import *
from beantop.charreader import CharReader
    
class Test(unittest.TestCase):
    def setUp(self):
        self.os=MockOs()
        self.file=MockFile()
        self.sys=MockSys(self.file)
        self.termios=MockTermios()
        self.fcntl=MockFcntl()
        self.char_reader = CharReader(self.os,  self.sys,  self.termios,  self.fcntl)

    def test_getchGetsNone(self):
        c = self.char_reader.get_char()
        self.assertEquals(None, c)

    def test_getchGetsException(self):
        c = self.char_reader.get_char()
        self.assertEquals(None, c)

    def test_getchGetsChar(self):
        self.file.char='a'
        c = self.char_reader.get_char()
        self.assertEquals('a', c)

    def test_setup_terminal_for_char_read(self):
        self.char_reader.setup_terminal_for_char_read()
        
    def test_reset_terminal_options(self):
        self.char_reader.reset_terminal_options()


if __name__ == "__main__":
    unittest.main() 



class Console:
    def __init__(self,  time,  char_reader,  screen_printer,  screen):
        self._time = time
        self._char_reader = char_reader
        self._screen_printer = screen_printer
        self._screen = screen
        self._finished = False

    def main_loop(self):
        try:
            self._char_reader.setup_terminal_for_char_read()
            while not self._finished:
                self._run_loop_once()
        finally:
            self._char_reader.reset_terminal_options()

    def _run_loop_once(self):
        time_limit = self._time.gmtime()+5
        scr = self._screen.render_screen()
        self._screen_printer.clear()
        time = self._time.get_printable_time()
        self._screen_printer.print_lines([time]+scr)
        self._loop_finished=False
        while self._time.gmtime()<time_limit and not self._loop_finished:
            self._process_char_queue()
            self._time.sleep(0.1)

    def _process_char(self):
        char_read = self._char_reader.get_char()
        if char_read is None:
            return False
        if char_read == 'q': 
            self._finished = True
            self._loop_finished=True
        if char_read == 'r': 
            self._loop_finished=True
        return True
       
    def _process_char_queue(self):
        found_char = self._process_char()
        while found_char and not self._finished:
            found_char = self._process_char()




HEADER_WIDTH = 25
FIELD_WIDTH = 15
GLOBAL_STATS_FIELDS = ["current-jobs-ready", 
                                "current-waiting", 
                                "pid",
                                "current-workers"]
TUBE_STATS_FIELDS = ["current-jobs-delayed", 
                            "current-jobs-ready", 
                            "current-jobs-reserved", 
                            "current-waiting", 
                            "current-watching"]


class BeanstalkdStats:
    def __init__(self, beanstalkd):
        self._beanstalkd = beanstalkd

    def render_screen(self):
        status = self._fetch_global_stats()
        tube_stats = self._fetch_tube_stats()
        return status+tube_stats
        
    def _fetch_global_stats(self):
        return self._format(
            self._beanstalkd.yaml_data_filtered("stats", GLOBAL_STATS_FIELDS))
    
    @staticmethod
    def _format(dic):
        ret_data = []
        for field, value in dic.iteritems():
            ret_data.append(field+": "+str(value))
        return ret_data        
                
    def _fetch_tube_stats(self):
        all_tubes = self._get_filtered_tubes()
        t_data = []
        max_items = 5
        remaining_tubes = all_tubes
        while len(remaining_tubes)>0:
            tubes_in_row = dict(remaining_tubes.items()[:max_items])
            remaining_tubes = dict(remaining_tubes.items()[max_items:])
            t_data += self._render_row(tubes_in_row)
        return t_data
        
    def _get_filtered_tubes(self):
        all_tubes = self._get_tubes()
        filtered_tubes = dict()
        for tube_name in all_tubes:
            tube = self._beanstalkd.yaml_data_filtered("stats-tube "+tube_name, 
                                                   TUBE_STATS_FIELDS)
            if (self._tube_has_meaningful_info(tube)):
                filtered_tubes[tube_name] = tube
        return filtered_tubes

    def _get_tubes(self):
        return self._beanstalkd.yaml_data("list-tubes")

    @staticmethod
    def _tube_has_meaningful_info(tube):
        return (tube["current-jobs-delayed"]+
                    tube["current-jobs-reserved"]+
                    tube["current-jobs-ready"])>0

    @staticmethod
    def _render_row(tubes_in_row):
        t_data = []
        t_headers = "Tube".ljust(HEADER_WIDTH)
        for tube_name, tube_data in tubes_in_row.iteritems(): 
            t_headers += tube_name.ljust(FIELD_WIDTH)
        t_data.append(t_headers)
        for field in TUBE_STATS_FIELDS: 
            t_row = field.rjust(HEADER_WIDTH)
            for tube_name, tube_data in tubes_in_row.iteritems(): 
                t_row += str(tube_data[field]).rjust(FIELD_WIDTH)
            t_data.append(t_row)
        return t_data





class ScreenPrinter:    
    def __init__(self,  operative_system,  sys):
        self._operative_system = operative_system
        self._sys = sys
                  
    def print_lines(self, lines):
        for line in lines:
            self._sys.stdout.write(line+'\n')
    
    def clear(self):
        self._operative_system.system('clear')




DEFAULT_HOST = 'localhost'
DEFAULT_PORT = 11300

class Arguments:
    def __init__(self, system, getopt):
        self._system = system
        self._getopt = getopt
        
    def process(self, argv):
        try:
            return self._process_arguments(argv)
        except self._getopt.GetoptError:
            self._exception_when_processing(argv)
            
    def _process_arguments(self, argv):
        opts, _ = self._getopt.getopt(argv, "h:p:", [])
        host = DEFAULT_HOST
        port = DEFAULT_PORT
        for opt, arg in opts:
            if opt == '-h':
                host = arg
            if opt == '-p':
                port = arg
        return host, port            
    
    def _exception_when_processing(self, argv):
        self._system.stdout.write(argv[0]+' -h <host> -p <port>')
        self._system.exit(2)




import unittest

from beantop.arguments import *
from mock import *

CUSTOM_PORT = 1234
CUSTOM_HOST='thehost'

class MockError(Exception):
    pass

class Test(unittest.TestCase):
    def setUp(self):
        self.sys=Mock(stdout=MagicMock())
        self.opts=MagicMock()
        self.getopt=Mock(getopt=self.opts, GetoptError=MockError)
        self.arguments = Arguments(self.sys, self.getopt)
        
    def test_parseNoArguments(self):
        self.opts.return_value=[], None

        connection_params = self.arguments.process(['test'])
        
        self.assertOutput([])
        self.assertEquals((DEFAULT_HOST, DEFAULT_PORT),  connection_params)        
        self.assertNotAppClosed()

    def test_parseIncompleteArguments(self):
        self.opts.side_effect=MockError
        
        connection_params = self.arguments.process(['test'])
        
        self.assertOutput(['test -h <host> -p <port>'])
        self.assertAppClosed()
        
    def test_parseOnlyPort(self):
        self.opts.return_value=[('-p', CUSTOM_PORT)], None
        
        connection_params = self.arguments.process(['test'])
        
        self.assertOutput([])
        self.assertEquals((DEFAULT_HOST, CUSTOM_PORT),  connection_params)        
        self.assertNotAppClosed()
    
    def test_parseOnlyHost(self):
        self.opts.return_value=[('-h', CUSTOM_HOST)], None
        
        connection_params = self.arguments.process(['test'])
        
        self.assertOutput([])
        self.assertEquals((CUSTOM_HOST, DEFAULT_PORT),  connection_params)
        self.assertNotAppClosed()
    
    def test_parseAllArguments(self):
        self.opts.return_value=[('-h', CUSTOM_HOST),('-p', CUSTOM_PORT)], None
        
        connection_params = self.arguments.process(['test'])
        
        self.assertOutput([])
        self.assertEquals((CUSTOM_HOST, CUSTOM_PORT),  connection_params)
        self.assertNotAppClosed()
        
    def assertAppClosed(self):
        self.assertAppClosedIs(True)
    
    def assertNotAppClosed(self):
        self.assertAppClosedIs(False)
    
    def assertAppClosedIs(self, state):
        self.sys.exit.assert_was_called(state)
        
    def assertOutput(self, lines):
        calls = map(call,  lines)
        self.sys.stdout.write.assert_has_calls(calls)

if __name__ == "__main__":
    unittest.main() 



import unittest

from beantop.factory import *

class Test(unittest.TestCase):
    def test_create_arguments_parser(self):
        create_arguments_parser()
    def test_start_application(self):
        start_application('', '')

    
if __name__ == "__main__":
    unittest.main() 



class MockTelnet:
    def __init__(self):
        self.openCalled=False
        
    def open(self, host, port):
        self.openCalled=True
        
    def write(self,  message):
        self.lastMessage=message
        
    def read_until(self,  chars):
        if self.lastMessage == 'stats\r\n':
            return    ("current-jobs-ready: 5\n"
                         "pid: 1223\n"
                         "current-waiting: 12\n"
                         "current-workers: 4\n")
        elif self.lastMessage == 'list-tubes\r\n':
            return    ("data:\n")
        elif self.lastMessage == 'stats-tube data\r\n':
            return    ("current-jobs-delayed: 4\n"
                         "current-jobs-ready: 10\n"
                         "current-waiting: 6\n"
                         "current-watching: 12\n"
                         "current-jobs-reserved: 3\n")
        else:
            raise Exception("Unknown message: "+repr(self.lastMessage))

class MockOs:
    O_NONBLOCK=True
    def system(self,  message):
        pass
    
class MockFile():
    def __init__(self):
        self.char=None
        self.written=[]
        
    def fileno(self):
        return 1
        
    def read(self,  a):
        if self.char is None:
            raise IOError
        return self.char        
        
    def write(self, str):
        self.written.append(str)        
    
class MockSys:
    def __init__(self, file):
        self.stdin=file
        self.stdout =file
    
class MockTermios:
    ICANON=False
    ECHO=False
    TCSANOW=False
    TCSAFLUSH=True
    def tcgetattr(self, a):
        return [1, 2, 3, 4]
    def tcsetattr(self, a, b, c):
        pass
    
class MockFcntl:
    F_GETFL=True
    F_SETFL=True
    def fcntl(self, a, b, c=True):
        return True
        
        
class MockTime:
    def strftime(self, format, t):
        return 'timestring'
    def gmtime(self):
        return (1234, 123, 1234)
    def sleep(self,  secs):
        self.sleepSecs=secs
    def time(self):
        return 1234

class MockScreen:
    printcalls=0
    def render_screen(self):
        self.printcalls+=1
        return [""]

class MockIO:
    totalsleep=0
    def put_char(self, ch):
        self.ch=ch
    def get_char(self):
        if len(self.ch)>0:
            ch = self.ch[0]
            self.ch = self.ch[1:]
            return ch
        else:
            return None
    def gmtime(self):
        return self.totalsleep
    def get_printable_time(self):
        return ""
    def clear(self):
        pass
    def print_lines(self,  lines):
        pass
    def sleep(self,  seconds):
        self.totalsleep+=1
        pass
    def setup_terminal_for_char_read(self):
        pass
    def reset_terminal_options(self):
        pass

    



from . import factory 
import sys

def main():
    arguments = factory.create_arguments_parser()
    host, port = arguments.process(sys.argv[1:])
    beanstalkd,  console = factory.start_application(host, port)
    beanstalkd.connect()
    console.main_loop()




class CharReader:
       
    def __init__(self,  operative_system, sys, termios, fcntl):
        self._operative_system = operative_system
        self._sys = sys
        self._termios = termios
        self._fcntl = fcntl
        self._stdin_file_descriptor = None
        self._oldterm = None
        self._oldflags = None
    
    def get_char(self):
        try:
            char_read = self._sys.stdin.read(1)
        except IOError: 
            char_read = None
        return char_read

    def setup_terminal_for_char_read(self):
        self._stdin_file_descriptor = self._sys.stdin.fileno()
        self._oldterm = self._termios.tcgetattr(self._stdin_file_descriptor)
        newattr = self._termios.tcgetattr(self._stdin_file_descriptor)
        newattr[3] = newattr[3] & ~self._termios.ICANON & ~self._termios.ECHO
        self._termios.tcsetattr(self._stdin_file_descriptor
                               , self._termios.TCSANOW, newattr)
        self._oldflags = self._fcntl.fcntl(self._stdin_file_descriptor
                                         , self._fcntl.F_GETFL)
        newflags = self._oldflags | self._operative_system.O_NONBLOCK
        self._fcntl.fcntl(self._stdin_file_descriptor
                         , self._fcntl.F_SETFL, newflags)
    
    def reset_terminal_options(self):
        self._termios.tcsetattr(self._stdin_file_descriptor
                               , self._termios.TCSAFLUSH, self._oldterm)
        self._fcntl.fcntl(self._stdin_file_descriptor
                         , self._fcntl.F_SETFL, self._oldflags)    



import unittest

from beantop.beanstalkdstats import BeanstalkdStats

class MockBeanstalkd:
    printcalls=0
    def yaml_data(self,  message):
        return ["tube_1",  "tube_2"]
    def yaml_data_filtered(self,  message,  fields):
        if message=="stats":
            return {"current-jobs-ready":1, "current-waiting":2, "current-workers":3, "pid": 1223}
        else:
            return {"current-jobs-delayed":1, 
                    "current-jobs-reserved":2, 
                    "current-jobs-ready":3, 
                    "current-waiting":12, 
                    "current-watching":32}


class Test(unittest.TestCase):
    def setUp(self):
        self.beanstalkd=MockBeanstalkd()
        self.beanstalkdstats = BeanstalkdStats(self.beanstalkd)
        
    def test_renderScreen(self):
        stats=self.beanstalkdstats.render_screen()
        self.assertEquals(
            ['current-jobs-ready: 1',
             'current-waiting: 2',
            'pid: 1223',             
             'current-workers: 3',
             'Tube                     tube_1         tube_2         ',
             '     current-jobs-delayed              1              1',
             '       current-jobs-ready              3              3',
             '    current-jobs-reserved              2              2',
             '          current-waiting             12             12', 
             '         current-watching             32             32'], stats)

if __name__ == "__main__":
    unittest.main() 



import unittest

from mocks import *
from beantop.clock import Clock
    
class Test(unittest.TestCase):
    def setUp(self):
        self.mock_time = MockTime()
        self.time = Clock(self.mock_time)

    def test_get_printable_time(self):
        t = self.time.get_printable_time()
        self.assertEquals('timestring', t)
        
    def test_gmtime(self):
        t = self.time.gmtime()
        self.assertEquals(1234, t)

    def test_sleep(self):
        self.time.sleep(10)
        self.assertTrue(10,  self.mock_time.sleepSecs)
    
if __name__ == "__main__":
    unittest.main() 



import unittest    

from mocks import *
from beantop.beanstalkdstats import BeanstalkdStats
from beantop.beanstalkd import Beanstalkd
from beantop.console import Console
from beantop.clock import Clock
from beantop.charreader import CharReader
from beantop.screenprinter import ScreenPrinter
from beantop.arguments import Arguments

class Test(unittest.TestCase):
    def setUp(self):
        self.mock_telnet = MockTelnet()
        self.mock_os = MockOs()
        self.mock_file=MockFile()        
        self.mock_file.char='q'
        self.mock_time = MockTime()        
        self.mock_sys = MockSys(self.mock_file)
        self.mock_termios = MockTermios()
        self.mock_fcntl = MockFcntl()
        
        self.beanstalkd = Beanstalkd(self.mock_telnet, '', '')
        self.stats = BeanstalkdStats(self.beanstalkd)
        self.char_reader = CharReader(self.mock_os, self.mock_sys, self.mock_termios, self.mock_fcntl)
        self.screen_printer = ScreenPrinter(self.mock_os, self.mock_sys)
        self.console = Console(Clock(self.mock_time),  self.char_reader,  self.screen_printer, self.stats)       

    def test_integration(self):
        self.beanstalkd.connect()
        self.console.main_loop()
    
if __name__ == "__main__":
    unittest.main() 



import unittest

from beantop.beanstalkd import Beanstalkd
from mocks import MockTelnet

class Test(unittest.TestCase):
    def setUp(self):
        self.telnet = MockTelnet()
        self.beanstalkd = Beanstalkd(self.telnet, 'thehost', 9057)
                        
    def test_connect(self):
        self.beanstalkd.connect()
        self.assertTrue(self.telnet.openCalled)        
        
    def test_yaml_data(self):
        line = self.beanstalkd.yaml_data("stats")
        self.assertEquals({'current-jobs-ready': 5, 'current-waiting': 12, 'current-workers': 4,  'pid':1223}, line)
        
    def test_yaml_data_filtered(self):
        line = self.beanstalkd.yaml_data_filtered("stats",  ["current-jobs-ready",  "current-waiting"])
        self.assertEquals({'current-jobs-ready': 5, 'current-waiting': 12}, line)

if __name__ == "__main__":
    unittest.main() 



import os
from setuptools import setup

version = '0.2.3'
README = os.path.join(os.path.dirname(__file__), 'README.md')
long_description = open(README).read() + 'nn'

setup(name='beantop', version=version, 
    author='alexsiri7',
    author_email='alexsiri7@gmail.com',
    license='GPL',
    long_description=long_description,
    url='https://github.com/alexsiri7/beantop', 
    classifiers=[
        "Development Status :: 4 - Beta",
        "Environment :: Console",
        "Programming Language :: Python", 
        "Intended Audience :: System Administrators", 
        "Topic :: Utilities"], 
    scripts=['scripts/beantop'],        
    )
